<?php

namespace Tvip\CoreBundle\Controller\ProviderApi;

use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\View\View;
use Symfony\Component\HttpFoundation\Request;
use Tvip\CoreBundle\Entity\Device;
use FOS\RestBundle\Controller\Annotations as Rest;
use Tvip\CoreBundle\Util\UniqueIdConverter;

class DeviceController extends BaseRestController
{
    /**
     * Get the list of devices

     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="provider", requirements="\d+", description="Filter by provider")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="unique_id", requirements="[0-9a-f:]+", description="Filter by mac address")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="start", requirements="\d+", default="0", description="Start item")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="limit", requirements="\d+", default="10", description="Number of items")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="sort", array=true, requirements="ASC|DESC", nullable=true, strict=true, description="Order by array (key is field, value is direction)")
     *
     *
     * @param ParamFetcher $paramFetcher
     *
     * @return mixed
     */
    public function cgetAction(ParamFetcher $paramFetcher){

        return parent::cgetAction($paramFetcher);
    }
    
    /**
     * @inheritdoc
     */
    public function postAction(Request $request)
    {
        $request = $this->hookRequest($request);
        return parent::postAction($request);
    }

    /**
     * @inheritdoc
     */
    public function getAction($id){
        return parent::getAction($id);
    }

    /**
     * @inheritdoc
     */
    public function putAction($id, Request $request){

        $request = $this->hookRequest($request);
        return parent::putAction($id,$request);
    }

    /**
     * @inheritdoc
     */
    public function deleteAction($id){
        return parent::deleteAction($id);
    }

    /**
     * @param $id int
     * @return View
     */
    public function getSubscriptionsAction($id){
        /**
         * @var $entity Device
         */
        $entity = $this->getEntityById($id);
        return $this->view(array('data'=>$entity->getCurrentDeviceSubscriptions()));
    }


    private function hookRequest(Request $request){
        if($json = json_decode($request->getContent())){
            $content = $request->getContent();
            if(property_exists($json,'unique_id')){
                $json->unique_id = UniqueIdConverter::convert($json->unique_id);
                $json->device_type = null;
                $content = json_encode($json);
            }
            $request->initialize(
                $request->query->all(),
                $request->request->all(),
                $request->attributes->all(),
                $request->cookies->all(),
                $request->files->all(),
                $request->server->all(),
                $content
            );
        }
        return $request;
    }
}